Kernel

Struct Kernel 

pub struct Kernel { /* private fields */ }
A loaded, compiled GPU kernel.

Can be launched on the GPU.

The #[kernel] macro adds a launch function that takes a &LaunchConfig as first argument and all kernel arguments afterwards.

Implementations

Source

impl Kernel

pub fn func(&self) -> hipFunction_t

Get the raw kernel function.