macro_rules! kernel_lib {
() => { ... };
}Expand description
The kernel_lib!() macro declares a crate as a library of GPU kernels.
It compiles the crate for the GPU and includes the compiled binary on the CPU side.
Example
// Somewhere at the top-level of your crate
gpu_kernel::kernel_lib!();